#ifndef HELPDLL_INCLUDED
#define HELPDLL_INCLUDED
/*---------------------------------------------------------------------------|
|                                                                            |
|  HELPDLL.H                                                                 |
|                                                                            |
|  Copyright (C) Microsoft Corporation 1990.                                 |
|  All Rights reserved.                                                      |
|                                                                            |
------------------------------------------------------------------------------
|                                                                            |
|  Module Intent                                                             |
|                                                                            |
|  Exports Winhelp's DLL macro binding functions (for LDLLHandler).          |
|                                                                            |
----------------------------------------------------------------------------*/
/***************************************************************************
 *  RegisterRoutine Prototypes.
 *
 *     Syntax of the proto-type is as follows:
 *       proto           ::= [parmlist]
 *       parmlist        ::= NULL OR [params]
 *       params          ::= [param] OR ([param] [params])
 *       param           ::= "i" OR "u" OR "s" OR "I" OR "U" OR "S"
 *       rettype         ::= [param] OR "v"
 *
 *     Example:          "uSiS"
 *                       ""
 *                       "uSs"
 * 
 *               Short Signed   'i'
 *               Long Signed    'I'
 *               Short Unsigned 'u'
 *               Long Unsigned  'U'
 *               Near String    's'
 *               Far  String     'S'
 *               Void           'v'
 *
 *  Custom macros may be embedded.  There are coding rules.			
 *  Embedding macro calls:      Syntax of the string is as follows:
 *       list            ::= NULL OR [macrolist]
 *       macrolist       ::= [macro] OR ([macro] ":" [macrolist])
 *       macro           ::= [name] "(" [paramlist] ")"
 *       name            ::= (ALPHA OR "_") [namechars]
 *       namechars       ::= NULL OR ALPHANUM OR "_"
 *       paramlist       ::= NULL OR [params]
 *       params          ::= [param] OR ([param] "," [params])
 *       param           ::= [posint] OR [int] OR [string] OR [macro] OR [var
 *       posint          ::= "0"..."9"
 *       int             ::= ("-" ([posint] OR [macro])) OR [posint]
 *       string          ::= (""" CHARS """) OR ("`" CHARS "'")
 *       var             ::= "hwndContext" OR "qchPath" OR "qError"
 *
 *   Example:        call1(5, "string", -call2()):call3("string")
 *                              call1(call1(call1(0))):call2()
 *
 *
 ***************************************************************************/
//
/* Macro error structures and error code definitions */
extern "C"
	{
	#define wMACRO_ERROR    128             /* Maximum length of an error msz   */
	
	                                        /* NOTE:  These #defines must be    */
	                                        /*   ordered because they are used  */
	                                        /*   as indexes into rgmpWMErrWErrs */
	#define wMERR_NONE           0          /* No error                         */
	#define wMERR_MEMORY         1          /* Out of memory (local)            */
	#define wMERR_PARAM          2          /* Invalid parameter passed         */
	#define wMERR_FILE           3          /* Invalid file parameter           */
	#define wMERR_ERROR          4          /* General macro error              */
	#define wMERR_MESSAGE        5          /* Macro error with message         */
	
	/* Flags constants indicating how errors *MAY* be handled. */
	#define fwMERR_ABORT    0x0001          /* Allow the "abort" option.        */
	#define fwMERR_CONTINUE 0x0002          /* Allow the "continue" option.     */
	#define fwMERR_RETRY    0x0004          /* Allow the "retry" option.        */
	
	/**********
	**
	**  The Macro Error structure is used to allow a macro to return error
	**  information.  It allows the macro to not only return pre-defined
	**  errors, but also to use the error string provided to pass back a
	**  customized error string.
	**
	*********/
	
	typedef struct
	  {                                     /* Contains flags indicating how an */
	                                        /*   error will be handled -- init- */
	  WORD  fwFlags;                        /*   ially set to fwMERR_ABORT      */
	                                        /* Error number if one occurs --    */
	  WORD  wError;                         /*   initially set to wMERR_NONE.   */
	                                        /* If wError == wMERR_MESSAGE, this */
	                                        /*   array will contain the error   */
	  char  rgchError[wMACRO_ERROR];        /*   message to be displayed.       */
	  } ME, NEAR * PME,  FAR * QME;
	
	/* Classes of messages that may be sent to DLLs */
	/* (legal returns for DW_WHATMSG handler in LDLLHandler routine). */
	#define DC_NOMSG     0x00000000         /* Classes of messages that may be  */
	#define DC_MINMAX    0x00000001         /*   send to DLLs                   */
	#define DC_INITTERM  0x00000002
	#define DC_JUMP      0x00000004
	#define DC_ACTIVATE  0x00000008
	#define DC_CALLBACKS 0x00000010
	
	#define DW_NOTUSED   0			// Messages sent to DLLs.
	#define DW_WHATMSG   1
	#define DW_MINMAX    2
	#define DW_SIZE      3
	#define DW_INIT      4
	#define DW_TERM      5
	#define DW_STARTJUMP 6
	#define DW_ENDJUMP   7
	#define DW_CHGFILE   8
	#define DW_ACTIVATE  9
	#define	DW_CALLBACKS 10
	
	/* Embedded Window messages */
	#define	EWM_RENDER		0x706A
	#define	EWM_QUERYSIZE		0x706B
	#define	EWM_ASKPALETTE		0x706C
	#define	EWM_FINDNEWPALETTE	0x706D
	
	/* DLLs should process this if they want to their audio to reset
	   correctly when another audio device attempts to use audio. */
	#define	WM_AUDIORESET	0x706E
	
	#define	HLPMENUEDITCOPY		0x0002
	
	/* Embedded window structure */
	typedef struct tagCreateInfo
	{
	  short   idMajVersion;
	  short   idMinVersion;
	  LPSTR   szFileName;		// Current Help file
	  LPSTR   szAuthorData;		// Text passed by the author
	  HANDLE  hfs;			// Handle to the current file system
	  DWORD   coFore;		// Foreground color for this topic
	  DWORD   coBack;		// Background color for this topic
	  } EWDATA, FAR *QCI;

	/* Embedded window rendering info */
	typedef struct tagRenderInfo
	{
		RECT rc;
		HDC hdc;
	}  RENDERINFO, FAR *QRI;
	
	/* file mode flags */
	#define fFSReadOnly       (BYTE)0x01  /* file (FS) is readonly                */
	#define fFSOpenReadOnly   (BYTE)0x02  /* file (FS) is opened in readonly mode */
	#define fFSReadWrite      (BYTE)0x00  // file (FS) is readwrite
	#define fFSOpenReadWrite  (BYTE)0x00  // file (FS) is opened in read/write mode
	
	/* seek origins */
	#define wFSSeekSet      0
	#define wFSSeekCur      1
	#define wFSSeekEnd      2
	
	/* low level info options */
	#define wLLSameFid    0
	#define wLLDupFid     1
	#define wLLNewFid     2
	
	/* Return codes */
	#define rcSuccess       0
	#define rcFailure       1
	#define rcExists        2
	#define rcNoExists      3
	#define rcInvalid       4
	#define rcBadHandle     5
	#define rcBadArg        6
	#define rcUnimplemented 7
	#define rcOutOfMemory   8
	#define rcNoPermission  9
	#define rcBadVersion    10
	#define rcDiskFull      11
	#define rcInternal      12
	#define rcNoFileHandles 13
	#define rcFileChange    14
	#define rcTooBig        15
	
	// following not from core engine:
	#define rcReadError     101
	
	/* Errors for Error() */
	#define wERRS_OOM                   2   /* Out of memory                    */
	#define wERRS_NOHELPPS              3   /* No help during printer setup     */
	#define wERRS_NOHELPPR              4   /* No help while printing           */
	#define wERRS_FNF                1001   /* Cannot find file                 */
	#define wERRS_NOTOPIC            1002   /* Topic does not exist             */
	#define wERRS_NOPRINTER          1003   /* Cannot print                     */
	#define wERRS_PRINT              1004
	#define wERRS_EXPORT             1005   /* Cannot copy to clipboard         */
	#define wERRS_BADFILE            1006
	#define wERRS_OLDFILE            1007
	#define wERRS_Virus              1011   /* Bad .EXE                         */
	#define wERRS_BADDRIVE           1012   /* Invalid drive                    */
	#define wERRS_WINCLASS           1014   /* Bad window class                 */
	#define wERRS_BADKEYWORD         3012   /* Invalid keyword                  */
	#define wERRS_BADPATHSPEC        3015   /* Invalid path specification       */
	#define wERRS_PATHNOTFOUND       3017   /* Path not found                   */
	#define wERRS_DIALOGBOXOOM       3018   /* Insufficient memory for dialog   */
	#define wERRS_DiskFull           5001   /* Disk is full                     */
	#define wERRS_FSReadWrite        5002   /* File read/write failure          */
	
	/* Actions for LGetInfo()  */
	#define GI_NOTHING   0                  /* Not used.                        */
	#define GI_INSTANCE  1                  /* Application instance handle      */
	#define GI_MAINHWND  2                  /* Main window handle               */
	#define GI_CURRHWND  3                  /* Current window handle            */
	#define GI_HFS       4                  /* Handle to file system in use     */
	#define GI_FGCOLOR   5                  /* Foreground color used by app     */
	#define GI_BKCOLOR   6                  /* Background color used by app     */
	#define GI_TOPICNO   7                  /* Topic number                     */
	#define GI_HPATH     8                  /* Handle containing path  -- caller*/
	
	/* Callback Function Table offsets: */
	#define HE_NotUsed               0
	#define HE_HfsOpenSz             1
	#define HE_RcCloseHfs            2
	#define HE_HfOpenHfs             3
	#define HE_RcCloseHf             4
	#define HE_LcbReadHf             5
	#define HE_LTellHf               6
	#define HE_LSeekHf               7
	#define HE_FEofHf                8
	#define HE_LcbSizeHf             9
	#define HE_FAccessHfs           10
	#define HE_RcLLInfoFromHf       11
	#define HE_RcLLInfoFromHfs      12
	#define HE_ErrorW               13
	#define HE_ErrorSz              14
	#define HE_GetInfo              15
	#define HE_API                  16
	
	typedef WORD  RC;                   /* Error return (return code)       */
	typedef HANDLE HFS;                 /* Handle to a file system          */
	typedef HANDLE HF;                  /* Handle to a file system bag file     */
	
	/* Function type definitions */

	typedef FARPROC FAR *VPTR;
	typedef HFS  (WINAPI *LPFN_HFSOPENSZ)( LPSTR szName, BYTE fMode );
	typedef HFS  (WINAPI *LPFN_RCCLOSEHFS)( HFS fs );
	typedef HF   (WINAPI *LPFN_HFOPENHFS) (HFS fs, LPSTR szName, BYTE bFlags);
	typedef RC   (WINAPI *LPFN_RCCLOSEHF) ( HF f );
	typedef LONG (WINAPI *LPFN_LCBREADHF) ( HF f, LPBYTE qb, LONG lcb );
	typedef LONG (WINAPI *LPFN_LTELLHF) ( HF f );
	typedef LONG (WINAPI *LPFN_LSEEKHF) ( HF f, LONG lOffset, WORD wOrigin );
	typedef BOOL (WINAPI *LPFN_FEOFHF) ( HF f );
	typedef LONG (WINAPI *LPFN_LCBSIZEHF) ( HF f );
	typedef BOOL (WINAPI *LPFN_FACCESSHFS) ( HFS fs, LPSTR szName, BYTE bFlags );
	typedef VOID (WINAPI *LPFN_ERRORW) ( int nError );
	typedef VOID (WINAPI *LPFN_ERRORSZ) ( LPSTR ErrMsg );
	typedef LONG (WINAPI *LPFN_LGETINFO) ( WORD wItem, HWND hwnd);
	typedef LONG (WINAPI *LPFN_FAPI) ( LPSTR qchHelp, WORD wCommand, DWORD ulData );
	typedef RC   (WINAPI *LPFN_RCLLINFOFROMHF) ( HF f, WORD wOption, WORD FAR * qFid, LONG FAR * qlBase, LONG FAR * qlcb );
	typedef RC   (WINAPI *LPFN_RCLLINFOFROMHFS) (HFS fs, LPSTR szName, WORD wOption, WORD FAR * qFid, LONG FAR * qlBase, LONG FAR * qlcb);

	LPFN_HFSOPENSZ         HfsOpenSz;
	LPFN_RCCLOSEHFS        RcCloseHfs;
	LPFN_HFOPENHFS         HfOpenHfs;
	LPFN_RCCLOSEHF         RcCloseHf;
	LPFN_LCBREADHF         LcbReadHf;
	LPFN_LTELLHF           LTellHf;
	LPFN_LSEEKHF           LSeekHf;
	LPFN_FEOFHF            FEofHf;
	LPFN_LCBSIZEHF         LcbSizeHf;
	LPFN_FACCESSHFS        FAccessHfs;
	LPFN_ERRORW            ErrorW;
	LPFN_ERRORSZ           ErrorSz;
	LPFN_LGETINFO          LGetInfo;
	LPFN_FAPI              FApi;
	LPFN_RCLLINFOFROMHF    RcLLInfoFromHf;
	LPFN_RCLLINFOFROMHFS   RcLLInfoFromHfs;
	
	BOOL GetCallBacks (VPTR, LONG);
}
#endif