// filedir.cpp : File and directory manipulation functions.
//
// Includes the following functions:
//		ChangeExtension
//		GetNameWithoutExtension
//
//

#include "stdafx.h"
#include "filedir.h"


//****************************************************************************
//	ChangeExtension
//
//		Changes the extension of a file to the specified extension
//
//	Parameters:
//
//	LPSTR cFile				Points to a null-terminated string containing the
//							current file name.
//	LPSTR cExtension		Points to a null-terminated string containing the
//							new extension.
//	LPSTR cNewFile			Points to a null-turminated string containing a file
//							name with the new value.							
//
//	Return value
//		BOOL				Always returns 1--doesn't really tell you anything.
//
//****************************************************************************
BOOL ChangeExtension (LPSTR cFile, LPSTR cExtension, LPSTR cNewFile)
{
	int success;
	int  ch = 46;
	LPSTR pdest;
	LPSTR lpSuccess;
	char dExtension[256];
	char string[256];

	pdest = strrchr( cExtension, ch );  //Does new extension have a dot?
 	if( pdest == NULL )	  //No dot found. Add to front of new extension.
 		{
		pdest = lstrcpy(dExtension, ".");  //Write a dot
		lpSuccess = lstrcat(dExtension, cExtension);  //Append the extension
		}
    else    //Dot found. Just copy the new extension.
		pdest = lstrcpy(dExtension, cExtension);
    
	pdest = lstrcpy(string, cFile);  //Copy the file name (don't destroy original)
	pdest = strrchr( string, ch );  //Does file have an extension?
	if( pdest != NULL )  //Extension found
	{
    	*(pdest) = '\0';   //Replace dot with NULL (strip old extension).
	}

	//Add new extension.
	lpSuccess = lstrcat(string, dExtension);  //Append new extension

	//Make lower case and return the new string
	success = wsprintf(cNewFile, "%s", _strlwr( _strdup( string ) ) );
	
	return 1;
}


//****************************************************************************
//	GetNameOnly
//
//		Gets the name (no path) of the executable--without its extension.
//
//	Parameters:
//
//	LPSTR cFile				Points to a null-terminated string containing the
//							file name, with or without a path.
//	LPSTR cNewFile			Points to a null-turminated string containing a file
//							name with the new value.							
//
//	Return value
//		BOOL				Always returns 1--doesn't really tell you anything.
//
//****************************************************************************
BOOL GetNameOnly (LPSTR cFile, LPSTR cNewFile)
{
	int success;
	int  ch;
	LPSTR pdest;
	char string[256];

	pdest = lstrcpy(string, cFile);  //Copy the file name (don't destroy original)
	
	//Remove the extension (if any)
	ch = 46;   //"."
	pdest = strrchr( string, ch );  //Does file have a dot?
	if( pdest != NULL )  //Dot found. There is an extension
	{
    	*(pdest) = '\0';   //Write NULL to character to replace dot (strip extension)
	}

	//Remove path (if any)
	ch=92;   //"\"
	pdest = strrchr( string, ch );  //Find the last backslash (if any)
	if (pdest == NULL)   //No path found. Return the whole string.
	{
		success = wsprintf(cNewFile, "%s", _strlwr( _strdup( string ) ) );   //Lower case
	}
	else    //Path found. Return only after the last backslash.
	{
		success = wsprintf(cNewFile, "%s", _strlwr( _strdup( pdest + 1 ) ) );   //Lower case
	}

	return 1;
}
